package arzumify.polyenergy.api;

// Provides energy to EnergyReceivers. Can be implemented by anything able to tick and provide energy!
public interface EnergyProvider extends EnergyPresence {
    /**
     * When called by a receiver, attempts to remove energy from the provider.
     * Do not attempt to extract energy until {@link EnergyReceiver#ready(EnergyProvider)} is called by the receiver.
     * The receiver may attempt to remove itself from the provider if it is no longer able to receive energy; this will be done via unready.
     *
     * @param amount   The target amount of energy to extract
     * @param receiver The receiver that is attempting to extract energy
     * @return The amount of energy actually extracted.
     */
    long extract(long amount, EnergyReceiver receiver);

    /**
     * When called by the server matchmaker, indicates that this is a possible receiver to call {@link EnergyReceiver#ready(EnergyProvider)} on.
     * This particular overload is for coordinate-based matchmaking.
     *
     * @param receiver         The receiver that exists.
     */
    void exists(EnergyReceiver receiver);
}